/******************************************************************************
* Copyright  2006 Altera Corporation, San Jose, California, USA.             *
* All rights reserved. All use of this software and documentation is          *
* subject to the License Agreement located at the end of this file below.     *
*******************************************************************************
*                                                                             *
* Filename:  network_utilities.c                                              *
*                                                                             *
* Taken from the Simple Socket Server example by Stephen O'Reilly.            *
******************************************************************************/

/* 
 * Remote Configuration example. 
 * 
 */

#include <alt_types.h>
#include <ctype.h>
#include <errno.h>
#include <stdio.h>
#include <sys/alt_flash.h>
#include "includes.h"
#include "io.h"
#include "system.h"
#include "remote_config.h"
#include "alt_error_handler.h"

/* Include headers to access eeprom if using Cyclone III NEEK. */

#ifdef PIO_ID_EEPROM_SCL_NAME
#include "alt_eeprom/alt_2_wire.h"
#include "alt_eeprom/alt_eeprom.h"
#endif


#include <alt_iniche_dev.h>

#include "ipport.h"
#include "tcpport.h"
#include "network_utilities.h"

#define IP4_ADDR(ipaddr, a,b,c,d) ipaddr = \
    htonl((((alt_u32)(a & 0xff) << 24) | ((alt_u32)(b & 0xff) << 16) | \
          ((alt_u32)(c & 0xff) << 8) | (alt_u32)(d & 0xff)))

/* Data Structure necessary for accessing the eeprom on the Cyclone III NEEK board. */

#ifdef PIO_ID_EEPROM_SCL_NAME
struct device devices[] = {
    { 0,    0x10,   0xA0,   0xA1    },
    { 8,    0x400,  0xA0,   0xA1    },
    { 16,   0x800,  0xA0,   0xA1    },
};

struct device *device = &(devices[0]);
#endif



/*
 * Handle to our Nios Development board LCD panel.
 */
FILE *LCDDevice;

int FindLastFlashSectorOffset(
    alt_u32                     *pLastFlashSectorOffset);

alt_u32 last_flash_sector_offset;
alt_u32 last_flash_sector;

/*
 * generate_and_store_mac_addr()
 * 
 * This routine is called when, upon program initialization, we discover
 * that there is no valid network settings (including MAC address) programmed
 * into flash memory at the last flash sector.  If it is not safe to use the 
 * contents of this last sector of flash, the user is prompted to
 * enter the serial number at the console.  A MAC address is then
 * generated using 0xFF followed by the last 2 bytes of the serial number 
 * appended to Altera's Vendor ID, an assigned MAC address range with the first
 * 3 bytes of 00:07:ED.  For example, if the Nios Development Board serial 
 * number is 040800017, the corresponding ethernet number generated will be
 * 00:07:ED:FF:8F:11.
 * 
 * It should be noted that this number, while unique, will likely differ from
 * the also unique (but now lost forever) MAC address programmed into the 
 * development board on the production line.
 * 
 * As we are erasing the entire flash sector, we'll re-program it with not
 * only the MAC address, but static IP, subnet, gateway, and "Use DHCP" 
 * sections. These fail-safe static settings are compatible with previous
 * Nios Ethernet designs, and allow the "factory-safe" design to behave 
 * as expected if the last flash sector is erased.
 */
error_t generate_and_store_mac_addr()
{
  error_t error = -1;
  alt_u32 ser_num = 0;
  char serial_number[9], flash_content[32];
  alt_flash_fd* flash_handle;
  int i = 0;
  
  printf("Can't read the MAC address from your board (this probably means\n");
  printf("that your flash was erased). We will assign you a MAC address and\n");
  printf("static network settings\n\n");
  
  while(!ser_num)
  {
    printf("Please enter your 9-digit serial number. This is printed on a \n");
    printf("label under your Nios dev. board. The first 3 digits of the \n");
    printf("label are ASJ and the serial number follows this.\n -->");
    
    for(i=0; i<9; i++)
    {
      serial_number[i] = getchar();
      putchar(serial_number[i]);
      
      /* Handle backspaces.  How civilized. */
      if ((serial_number[i] == 0x08) && (i >= 0)) 
      {
        i--;
      }
    }
    printf("\n");
            
    for(i=0; i<9; i++)
    {
      if (isdigit(serial_number[i]))
      {
        ser_num *= 10;
        ser_num += serial_number[i] - '0';
      }
      else
      {
        ser_num = 0;
        printf("Serial number only contains decimal digits and is non-zero\n");
        break;
      }
    }
    
    if (ser_num)
    {
      /* This says the image is safe */
      flash_content[0] = 0xfe;
      flash_content[1] = 0x5a;
      flash_content[2] = 0x0;
      flash_content[3] = 0x0;
      
      /* This is the Altera Vendor ID */
      flash_content[4] = 0x0;
      flash_content[5] = 0x7;
      flash_content[6] = 0xed;
      
      /* Reserverd Board identifier for erase boards */
      flash_content[7] = 0xFF;
      flash_content[8] = (ser_num & 0xff00) >> 8;
      flash_content[9] = ser_num & 0xff;

      /* Then comes a 16-bit "flags" field */
      flash_content[10] = 0xFF;
      flash_content[11] = 0xFF;
      
      /* Then comes the static IP address */
      flash_content[12] = IPADDR0;
      flash_content[13] = IPADDR1;
      flash_content[14] = IPADDR2;
      flash_content[15] = IPADDR3;
      
      /* Then comes the static nameserver address */
      flash_content[16] = 0xFF;
      flash_content[17] = 0xFF;
      flash_content[18] = 0xFF;
      flash_content[19] = 0xFF;
      
      /* Then comes the static subnet mask */
      flash_content[20] = MSKADDR0;
      flash_content[21] = MSKADDR1;
      flash_content[22] = MSKADDR2;
      flash_content[23] = MSKADDR3;
      
      /* Then comes the static gateway address */
      flash_content[24] = GWADDR0;
      flash_content[25] = GWADDR1;
      flash_content[26] = GWADDR2;
      flash_content[27] = GWADDR3;
      
      /* And finally whether to use DHCP - set all bits to be safe */
      flash_content[28] = 0xFF;
      flash_content[29] = 0xFF;
      flash_content[30] = 0xFF;
      flash_content[31] = 0xFF;
      
      /* Write the MAC address to flash */
      flash_handle = alt_flash_open_dev(EXT_FLASH_NAME);
      if (flash_handle)
      {
        alt_write_flash(flash_handle,
                        last_flash_sector_offset,
                        flash_content,
                        32);
        alt_flash_close_dev(flash_handle);
        error = 0;
      }
    }
  }
  return error;    
}

#ifdef PIO_ID_EEPROM_SCL_NAME /* Cyclone III NEEK Board. */
error_t get_board_mac_addr(unsigned char mac_addr[6],struct device *device)
{
  error_t error = 0;

  alt_u32 signature = 0;

  alt_two_wire s_eeprom;
  alt_two_wire* eeprom = &s_eeprom;

/* Initialize the EEPROM. */
  
  eeprom->scl_pio = PIO_ID_EEPROM_SCL_BASE;
  eeprom->sda_pio = PIO_ID_EEPROM_DAT_BASE;

  alt_2_wireInit(eeprom);
  alt_2_wireSetDelay(10);

  printf("EEPROM device 24LC%d size is %x\n", device->id, device->size);

  char *buf = malloc(10);

  if (!error)
  {
    if(!readRandom(device, 0, buf, 10)) {
      printf( "Successfully read 16 bytes from EEPROM.\n");
      signature = (buf[2]*0x10000 & 0xff0000) + (buf[3]*0x100 & 0xff00) + (buf[4] & 0xff);
    }
    printf( "Signature = 0x%x.\n", (unsigned int) signature);
    if (signature != 0x0007ed)
    {
      printf( "Your EEK LCD daughtercard is not programmed with a correct MAC Address!\n");
      printf( "Please contact Altera support for instructions on how to correct this problem!\n");
    }
  }
  
  if (!error)
  {
    mac_addr[0] = buf[2] & 0xFF;
    mac_addr[1] = buf[3] & 0xFF;
    mac_addr[2] = buf[4] & 0xFF;
    mac_addr[3] = buf[5] & 0xFF;
    mac_addr[4] = buf[6] & 0xFF;
    mac_addr[5] = buf[7] & 0xFF;
    
    printf("Your Ethernet MAC address is %02x:%02x:%02x:%02x:%02x:%02x\n", 
            mac_addr[0],
            mac_addr[1],
            mac_addr[2],
            mac_addr[3],
            mac_addr[4],
            mac_addr[5]);

  }
  free(buf);
  
  return error;
}
#else /* Every other board. */
/*
 * get_board_mac_addr
 *
 * Read the MAC address in a board specific way
 *
 */

error_t get_board_mac_addr(unsigned char mac_addr[6])
{
  error_t  error = 0;
  alt_u32 signature;

  /* Get the flash sector with the MAC address. */
  error = FindLastFlashSectorOffset(&last_flash_sector_offset);
  if (!error)
    last_flash_sector = EXT_FLASH_BASE + last_flash_sector_offset;
  else
    printf ("Could not locate flash sector with MAC address.\n");

/* This last_flash_sector region of flash is examined to see if 
 * valid network settings are present, indicated by a signature of 0x00005afe at 
 * the first address of the last flash sector.  This hex value is chosen as the 
 * signature since it looks like the english word "SAFE", meaning that it is 
 * safe to use these network address values.  
*/
  if (!error)
  {
    signature = IORD_32DIRECT(last_flash_sector, 0);
    if (signature != 0x00005afe)
    {
      error = generate_and_store_mac_addr();
    }
  }
  
  if (!error)
  {
    mac_addr[0] = IORD_8DIRECT(last_flash_sector, 4);
    mac_addr[1] = IORD_8DIRECT(last_flash_sector, 5);
    mac_addr[2] = IORD_8DIRECT(last_flash_sector, 6);
    mac_addr[3] = IORD_8DIRECT(last_flash_sector, 7);
    mac_addr[4] = IORD_8DIRECT(last_flash_sector, 8);
    mac_addr[5] = IORD_8DIRECT(last_flash_sector, 9);
    
    printf("Your Ethernet MAC address is %02x:%02x:%02x:%02x:%02x:%02x\n", 
            mac_addr[0],
            mac_addr[1],
            mac_addr[2],
            mac_addr[3],
            mac_addr[4],
            mac_addr[5]);

  }

  return error;
}
#endif
/*
* get_mac_addr
*
* Read the MAC address in a board specific way
*
*/
int get_mac_addr( NET net, unsigned char mac_addr[6] )
{
#ifdef PIO_ID_EEPROM_SCL_NAME
  return (get_board_mac_addr(mac_addr, device));
#else
  return (get_board_mac_addr(mac_addr));
#endif
}


/*******************************************************************************
 *
 * Flash service functions.
 *
 ******************************************************************************/

#include "sys/alt_flash.h"
#include "sys/alt_flash_dev.h"

/*
 * FindLastFlashSectorOffset
 *
 *   <-- pLastFlashSectorOffset Offset of last sector in flash.
 *
 *   This function finds the offset to the last sector in flash and returns it
 * in pLastFlashSectorOffset.
 */

int FindLastFlashSectorOffset(
    alt_u32                     *pLastFlashSectorOffset)
{
    alt_flash_fd                *fd;
    flash_region                *regions;
    int                         numRegions;
    flash_region                *pLastRegion;
    int                         lastFlashSectorOffset;
    int                         n;
    int                         error = 0;

    /* Open the flash device. */
    fd = alt_flash_open_dev(EXT_FLASH_NAME);
    if (fd <= 0)
        error = -1;

    /* Get the flash info. */
    if (!error)
        error = alt_get_flash_info(fd, &regions, &numRegions);

    /* Find the last flash sector. */
    if (!error)
    {
        pLastRegion = &(regions[0]);
        for (n = 1; n < numRegions; n++)
        {
            if (regions[n].offset > pLastRegion->offset)
                pLastRegion = &(regions[n]);
        }
        lastFlashSectorOffset =   pLastRegion->offset
                                + pLastRegion->region_size
                                - pLastRegion->block_size;
    }

    /* Return results. */
    if (!error)
        *pLastFlashSectorOffset = lastFlashSectorOffset;

    return (error);
}

/*
 * get_ip_addr()
 * 
 * In our system, we are either attempting DHCP auto-negotiation of IP address,
 * or we are setting our own static IP, Gateway, and Subnet Mask addresses our
 * self. This routine is where that happens.
 */
int get_ip_addr(alt_iniche_dev* p_dev, 
                ip_addr* ipaddr, 
                ip_addr* netmask, 
                ip_addr* gw,
                int* use_dhcp)
{
  int got_ip_addr = 0;  

#ifdef LCD_DISPLAY_NAME 
     LCDDevice = fopen(LCD_DISPLAY_NAME, "w");
     if (LCDDevice <= 0) 
     {
         alt_NetworkErrorHandler(EXPANDED_DIAGNOSIS_CODE,
                  "[network_utilities:get_ip_addr] fopen LCD_DISPLAY_NAME failed");
     }
#endif /* LCD_DISPLAY_NAME */

//  if (strcmp(p_dev->name, "/dev/" INICHE_DEFAULT_IF))
//    return( got_ip_addr );
    
#ifdef DHCP_CLIENT

    IP4_ADDR(*ipaddr, IPADDR0, IPADDR1, IPADDR2, IPADDR3);
    IP4_ADDR(*gw, GWADDR0, GWADDR1, GWADDR2, GWADDR3);
    IP4_ADDR(*netmask, MSKADDR0, MSKADDR1, MSKADDR2, MSKADDR3);
    
    *use_dhcp = 1;

#ifdef LCD_DISPLAY_NAME  
    /* Clear the LCD screen */
    fprintf(LCDDevice, "\x1b");
    fprintf(LCDDevice, "[2J");
    fprintf(LCDDevice, "Using DHCP to\nfind IP Addr");
#endif /* LCD_DISPLAY_NAME */
    printf("Using DHCP to find an IP Address\n");

#else /* not DHCP_CLIENT */
    IP4_ADDR(*ipaddr, IPADDR0,IPADDR1,IPADDR2,IPADDR3);
    IP4_ADDR(*gw, GWADDR0,GWADDR1,GWADDR2,GWADDR3);
    IP4_ADDR(*netmask, MSKADDR0,MSKADDR1,MSKADDR2,MSKADDR3);
    *use_dhcp = 0;

    printf("Static IP Address is %d.%d.%d.%d\n", 
        ip4_addr1(*ipaddr),
        ip4_addr2(*ipaddr),
        ip4_addr3(*ipaddr),
        ip4_addr4(*ipaddr));

#ifdef LCD_DISPLAY_NAME  
      /* Clear the LCD screen */
      fprintf(LCDDevice, "\x1b");
      fprintf(LCDDevice, "[2J");
      fprintf(LCDDevice, "Static IP Addr\n %d.%d.%d.%d",
        ip4_addr1(*ipaddr),
        ip4_addr2(*ipaddr),
        ip4_addr3(*ipaddr),
        ip4_addr4(*ipaddr));
#endif /* LCD_DISPLAY_NAME */

#endif /* not DHCP_CLIENT */

    got_ip_addr = 1;
    return (got_ip_addr);
}



/******************************************************************************
*                                                                             *
* License Agreement                                                           *
*                                                                             *
* Copyright (c) 2006 Altera Corporation, San Jose, California, USA.           *
* All rights reserved.                                                        *
*                                                                             *
* Permission is hereby granted, free of charge, to any person obtaining a     *
* copy of this software and associated documentation files (the "Software"),  *
* to deal in the Software without restriction, including without limitation   *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,    *
* and/or sell copies of the Software, and to permit persons to whom the       *
* Software is furnished to do so, subject to the following conditions:        *
*                                                                             *
* The above copyright notice and this permission notice shall be included in  *
* all copies or substantial portions of the Software.                         *
*                                                                             *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR  *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,    *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE *
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER      *
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING     *
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER         *
* DEALINGS IN THE SOFTWARE.                                                   *
*                                                                             *
* This agreement shall be governed in all respects by the laws of the State   *
* of California and by the laws of the United States of America.              *
* Altera does not recommend, suggest or require that this reference design    *
* file be used in conjunction or combination with any other product.          *
******************************************************************************/
